@@ -4,8 +4,8 @@
  +-----------------------------------------------------------------------+
  | program/include/iniset.php                                            |
  |                                                                       |
- | This file is part of the RoundCube Webmail client                     |
- | Copyright (C) 2008-2009, RoundCube Dev, - Switzerland                 |
+ | This file is part of the Roundcube Webmail client                     |
+ | Copyright (C) 2008-2009, Roundcube Dev, - Switzerland                 |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -16,25 +16,42 @@
  |         Thomas Bruederli <roundcube@gmail.com>                        |
  +-----------------------------------------------------------------------+
 
- $Id: iniset.php 3081 2009-10-31 13:20:02Z thomasb $
+ $Id: iniset.php 4469 2011-01-29 14:55:12Z thomasb $
 
 */
 
-
-// application constants
-define('RCMAIL_VERSION', '0.3.1');
-define('RCMAIL_CHARSET', 'UTF-8');
-define('JS_OBJECT_NAME', 'rcmail');
-
-if (!defined('INSTALL_PATH')) {
-  define('INSTALL_PATH', dirname($_SERVER['SCRIPT_FILENAME']).'/');
+// Some users are not using Installer, so we'll check some
+// critical PHP settings here. Only these, which doesn't provide
+// an error/warning in the logs later. See (#1486307).
+$crit_opts = array(
+    'mbstring.func_overload' => 0,
+    'suhosin.session.encrypt' => 0,
+    'session.auto_start' => 0,
+    'file_uploads' => 1,
+);
+foreach ($crit_opts as $optname => $optval) {
+    if ($optval != ini_get($optname)) {
+        die("ERROR: Wrong '$optname' option value. Read REQUIREMENTS section in INSTALL file or use Roundcube Installer, please!");
+    }
 }
 
-define('RCMAIL_CONFIG_DIR', INSTALL_PATH . 'config');
+// application constants
+define('RCMAIL_VERSION', '0.5.1');
+define('RCMAIL_CHARSET', 'UTF-8');
+define('JS_OBJECT_NAME', 'rcmail');
+define('RCMAIL_START', microtime(true));
+
+if (!defined('INSTALL_PATH')) {
+    define('INSTALL_PATH', dirname($_SERVER['SCRIPT_FILENAME']).'/');
+}
+
+if (!defined('RCMAIL_CONFIG_DIR')) {
+    define('RCMAIL_CONFIG_DIR', INSTALL_PATH . 'config');
+}
 
 // make sure path_separator is defined
 if (!defined('PATH_SEPARATOR')) {
-  define('PATH_SEPARATOR', (strtoupper(substr(PHP_OS, 0, 3)) == 'WIN') ? ';' : ':');
+    define('PATH_SEPARATOR', (strtoupper(substr(PHP_OS, 0, 3)) == 'WIN') ? ';' : ':');
 }
 
 // RC include folders MUST be included FIRST to avoid other
@@ -47,18 +64,10 @@ $include_path.= INSTALL_PATH . 'program/include' . PATH_SEPARATOR;
 $include_path.= ini_get('include_path');
 
 if (set_include_path($include_path) === false) {
-  die('Fatal error: ini_set/set_include_path does not work.');
+    die("Fatal error: ini_set/set_include_path does not work.");
 }
 
 ini_set('error_reporting', E_ALL&~E_NOTICE);
-if  (isset($_SERVER['HTTPS'])) {
-   ini_set('session.cookie_secure', ($_SERVER['HTTPS'] && ($_SERVER['HTTPS'] != 'off'))?1:0);
-} else {
-   ini_set('session.cookie_secure', 0);
-}
-ini_set('session.name', 'roundcube_sessid');
-ini_set('session.use_cookies', 1);
-ini_set('session.use_only_cookies', 1);
 
 // increase maximum execution time for php scripts
 // (does not work in safe mode)
@@ -66,8 +75,8 @@ ini_set('session.use_only_cookies', 1);
 
 // set internal encoding for mbstring extension
 if(extension_loaded('mbstring'))
-  mb_internal_encoding(RCMAIL_CHARSET);
-	      
+    mb_internal_encoding(RCMAIL_CHARSET);
+
 
 /**
  * Use PHP5 autoload for dynamic class loading
@@ -77,26 +86,33 @@ if(extension_loaded('mbstring'))
  */
 function rcube_autoload($classname)
 {
-  $filename = preg_replace(
-      array(
-        '/MDB2_(.+)/',
-        '/Mail_(.+)/',
-        '/Net_(.+)/',
-        '/^html_.+/',
-        '/^utf8$/',
-        '/html2text/'
-      ),
-      array(
-        'MDB2/\\1',
-        'Mail/\\1',
-        'Net/\\1',
-        'html',
-        'utf8.class',
-        'lib/html2text'  // see #1485505
-      ),
-      $classname
-  );
-  include $filename. '.php';
+    $filename = preg_replace(
+        array(
+            '/MDB2_(.+)/',
+            '/Mail_(.+)/',
+            '/Net_(.+)/',
+            '/Auth_(.+)/',
+            '/^html_.+/',
+            '/^utf8$/',
+        ),
+        array(
+            'MDB2/\\1',
+            'Mail/\\1',
+            'Net/\\1',
+            'Auth/\\1',
+            'html',
+            'utf8.class',
+        ),
+        $classname
+    );
+
+    if ($fp = @fopen("$filename.php", 'r', true)) {
+        fclose($fp);
+        include_once("$filename.php");
+        return true;
+    }
+
+    return false;
 }
 
 spl_autoload_register('rcube_autoload');
@@ -106,17 +122,15 @@ spl_autoload_register('rcube_autoload');
  */
 function rcube_pear_error($err)
 {
-  error_log(sprintf("%s (%s): %s",
-    $err->getMessage(),
-    $err->getCode(),
-    $err->getUserinfo()), 0);
+    error_log(sprintf("%s (%s): %s",
+        $err->getMessage(),
+        $err->getCode(),
+        $err->getUserinfo()), 0);
 }
 
-// include global functions
-require_once 'include/bugs.inc';
-require_once 'include/main.inc';
-require_once 'include/rcube_shared.inc';
-
-
 // set PEAR error handling (will also load the PEAR main class)
 PEAR::setErrorHandling(PEAR_ERROR_CALLBACK, 'rcube_pear_error');
+
+// include global functions
+require_once 'include/main.inc';
+require_once 'include/rcube_shared.inc';
